c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine clddiag(time,ndate,idebug)
c
c-----ATDM v1.00
c
c     Diagnoses cloud parameters (top, bottom, freezing level, cloud cover)
c     and type (stratiform vs. convective).  Diagnosis based on input 
c     temperature, moisture, and pressure profiles.  Convection triggered if 
c     moisture convergence is above a threshold value and sounding through
c     a column is convectively unstable; following MM5 (Anthes et al, NCAR
c     Tech Note TN-282+STR, 1987).
c
c-----95/05/19:
c     C Emery SAI
c-----95/10/04:
c     -put max limit on CS = 1.0
c     -put limit on CS such that if CS < 0.1, CS = 0.0
c-----95/10/18:
c     -restructured for more efficient handling of lwet and rain
c-----95/10/20:
c     -moved calculation of xmd to just after check of convective instability;
c      now checks for xmd > 0 for convection to occur
c     -checks for rain > 0.01 in/hr (instead of 0) to diagnose rain-clouds
c-----95/12/26:
c     -put min limit on kbot (>=3) for convective clouds at about line 185
c      (quick fix to try and prevent oscilation for strong convective
c       columns with low bottom)
c
c-----96/09/26:
c     -fixed argument type mismatch in scavrat call -- cd (real) passed 
c      as logical variable, fixed by defining lcd=.true. when cd .gt. 0.
c
c
c     Input arguments:
c               time:           model time    
c               ndate:          model date
c               idebug:         diagnostic debug flag
c     Files written:
c               6:              standard output file unit (*)
c     Routines called:
c               scavrat
c     Called by:
c               update
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'comuam.cmd'
      include 'advec.cmd'

      real lv
      logical lprnt,lcd
      dimension ti(mxz), pi(mxz), zi(mxz)
      dimension td(mxz), theta(mxz), qs(mxz), rh(mxz), rhc(mxz)
      dimension qi(mxz), thetae(mxz), cs(mxz), rate(mxz)
      dimension clw(mxz), rlw(mxz)

      data p0 /1000./, gamma /0.286/, e0 /6.11/, eps /0.622/
      data lv /2.5e6/, rv /461./, cp /1004./, g /9.8/, rd /287./
      nz = nlayer
      gamadry = g/cp
c
c-----Bit 14 (8192's location) turns on debug writes in clddiag
c
      idiag = and(idebug,8192)
      if (idiag.ne.0) write(*,*)'Enter CLDDIAG: ',time,ndate
c
      do 100 j = 1,noy
c
c-----check for outside of modeling region
c
        if (istr(j).ge.nox) go to 100
        i1 = istr(j)+1
        i2 = iend(j)-1

        do 200 i = i1,i2
          if (j.le.jstr(i)) go to 200
          if (j.ge.jend(i)) go to 200
          lprnt = .false.
	  lcd=.false.
          if (i.eq.45 .and. j.eq.51 .and. idiag.ne.0) lprnt = .true.
c          if (idiag.ne.0) lprnt = .true.
c
c-----Load variables for column
c
          cd = 0.
          csav = 0.
          xmd(i,j) = 0.
          ktop = 0
          kbot = 0
          kfreez = 0 
          do 10 k = 1,nz
            cs(k) = 0.
            zi(k) = h(i,j,k)
            pi(k) = press(i,j,k)
            ti(k) = temper(i,j,k)
            qi(k) = ch2o(i,j,k)
   10     continue
c         
c-----Loop over layers to find temperature and moisture parameters in layers
c
          if (lprnt) write(*,1001)
          k700 = 0
          rhav = 0.
          do 20 k = 1,nz
            if (pi(k).lt.700. .and. k700.eq.0) then
              k700 = k - 1
              if (k700.lt.1) k700 = 1
            endif
            theta(k) = ti(k)*(p0/pi(k))**gamma

            ev = qi(k)*pi(k)/(qi(k) + eps)
            tdk = 1./273 - (rv/lv)*alog(ev/e0)
            td(k) = 1./tdk
            es = e0*exp((lv/rv)*(1./273. - 1./ti(k)))
            qs(k) = (eps*es)/(pi(k) - es)
            rh(k) = 100.*ev/es

            rhav = rhav + rh(k)*(sigma(k) - sigma(k+1))
            prat = pi(k)/psfc(i,j)
            term1 = 100.*2.*prat*(1. - prat)
            term2 = 1. + sqrt(3.)*(prat - 0.5)
            rhc(k) = 100. - term1*term2

            if (lprnt) write(*,1000) sigma(k),sigma(k+1),pi(k),ti(k),
     &                 theta(k),qi(k),qs(k),td(k),rh(k),rhc(k)
   20     continue
 1000     format(2x,2f6.2,3(1x,f6.1),2(1x,f7.4),1x,f6.1,2(1x,f5.0))
 1001     format('     d(sigma)    p     T      TH      qi       qs',
     &           '     Td    RH    RHc')
          if (lprnt) write(*,*)'Column mean RH: ',rhav
          if (rhav.lt.50.) rhav = 50.
          bfac = 2.*(1. - rhav/100.)
c
c-----Loop over levels and determine the extent of stratiform clouds
c
          if (lprnt) write(*,*)'Layer   Cloud Fraction'
          csmax = 0.
          if (ti(1).le.273.) kfreez = 1
          do 30 k = 1,nz
            if (rh(k).gt.rhc(k)) then
              cs(k) = ((rh(k) - rhc(k))/(100. - rhc(k)))**2 
              cs(k) = amin1(cs(k),1.)
              if (cs(k).lt.0.1) cs(k) = 0.
            endif
            csmax = amax1(cs(k),csmax)
            if (kfreez.eq.0 .and. ti(k).lt.270.) kfreez = k
            if (lprnt) write(*,1002) k,100.*cs(k)
   30     continue
 1002     format(i5,7x,f10.1)
c
c-----If it is raining in this column, determine whether cloud type is
c     convective or stratiform
c
c          if (.not.lwet .or. rain(i,j).lt.0.01) goto 75
          if (.not.lwet) goto 75
c
c-----Loop over layers between surface and 700 mb
c     find layer having maximum equivalent potential temperature, and save
c     LCL (cloud base), T, and p at that level
c
          themax = 0.
          do 40 k = 1,k700
            gamaqs = (g*td(k)**2)/(eps*lv*ti(k))
            zs = (ti(k)-td(k))/(gamadry-gamaqs)
            tmps = ti(k) - gamadry*zs
            tbar = (tmps + ti(k))/2.
            dlnp = (g*zs)/(rd*tbar)
            prss = pi(k)*exp(-dlnp)
            thetae(k) = theta(k)*exp(lv*qi(k)/(cp*tmps))
            if (thetae(k).gt.themax) then
              if (k.eq.1) then
                zlcl = zi(k)/2. + zs
              else
                zlcl = zi(k-1) + (zi(k) - zi(k-1))/2. + zs
              endif
              tlcl = tmps
              plcl = prss
              themax = thetae(k)
            endif
   40     continue
          if (lprnt) then
            write(*,*)'LCL at temperature: ',tlcl
            write(*,*)'          pressure: ',plcl
            write(*,*)'            height: ',zlcl
            write(*,*)'           theta_e: ',themax
          endif
c
c-----Loop over layers from LCL to top; find saturated equivelent potential
c     temperature for each level, and convective cloud top, bottom, freezing 
c     level
c
          ipos = 0
          sum = 0.
          do 50 k = 1,nz
c          if (k.eq.1 .or. pi(k).gt.plcl) then
           if (k.le.2 .or. pi(k).gt.plcl) then
              kbot = k+1
              goto 50
            endif
            dsigma = sigma(k) - sigma(k+1)
            thetae(k) = theta(k)*exp(lv*qs(k)/(cp*ti(k)))
            if ( lprnt ) then
              write(*,*) 'k,ipos,themax,thetae ',k,ipos,
     &                     themax,thetae(k)
            endif
            if (ipos.eq.0 .and. themax.gt.thetae(k)) ipos = 1
            if (ipos.eq.1 .and. themax.le.thetae(k)) then
              ktop = k
              goto 51
            endif
            sum = sum + (themax - thetae(k))*dsigma
            if (lprnt) write(*,*)'k,themax,thetae,tdif,dsigma,sum',
     &                 k,themax,thetae(k),themax-thetae(k),dsigma,sum
   50     continue
          if (ipos.eq.1) then
            ktop = nz
          else
c
c-----No convective clouds; no positive energy
c
            if (lprnt) write(*,*)'Sounding is convectively stable'   
            if (lprnt) write(*,*)'No positive area: energy = ',sum
            ktop = 0
            kbot = 0
            goto 61
          endif
   51     if (sum.lt.0) then
            if (lprnt) write(*,*)'Sounding is convectively stable'
            if (lprnt) write(*,*)'energy = ',sum
            ktop = 0
            kbot = 0
            goto 61
          else
c
c-----Convective clouds possible; positive energy
c
            if ( lprnt) write(*,*) 'kbot,ktop :',kbot,ktop
            if ( lprnt) write(*,*) 'sigma_kbot,sigma_ktop+1:',
     &                               sigma(kbot),sigma(ktop+1)
            thsum = 0.
            thdfsum = 0.
            do 65 k = kbot,ktop
              dsigma = sigma(k) - sigma(k+1)
              thsum = thsum + thetae(k)*dsigma
              thdfsum = thdfsum + (thetae(kbot) - thetae(k))*dsigma
              if (lprnt ) then
                write(*,*) ' k,dsigma,thetae,thsum,thdfsum :',
     &           k,dsigma,thetae(k),thsum,thdfsum
              endif
   65       continue
            thtebar = thsum/(sigma(kbot) - sigma(ktop+1))
            xmd(i,j) = 1.e-3*(themax - thtebar)/(thdfsum+1.0e-10)
            if ( lprnt ) write(*,*) ' thtebar,xmd :',thtebar,xmd(i,j)
            if (xmd(i,j) .gt. 1.0) xmd(i,j) = 1.0
            if (xmd(i,j).lt.0.) then
              if (lprnt) write(*,*)'Convection not strong enough'
              if (lprnt) write(*,*)'xmd < 0: ',i,j,xmd(i,j)
              if (lprnt) write(*,*)'themax,thtebar,thdfsum',
     &                              themax,thtebar,thdfsum
              xmd(i,j) = 0.
              ktop = 0
              kbot = 0
              goto 61
            endif
            delsig = sigma(kbot) - sigma(ktop+1)
            if (delsig.lt.0.3) then
              if (lprnt) write(*,*)'Convection not deep enough'
              if (lprnt) write(*,*)'energy = ',sum
              ktop = 0
              kbot = 0
              goto 61
            else
              if (lprnt) then
                write(*,*)'Sounding is convective!'
                write(*,*)'energy = ',sum
                write(*,*)'Top at   : ',ktop,sigma(ktop+1)
                write(*,*)'Bottom at: ',kbot,sigma(kbot)
                write(*,*)'Freezing : ',kfreez
              endif
            endif
          endif
c
c-----If sounding is convectively unstable, calculate moisture convergence
c     through entire column and compare to critical value; then calculate 
c     fractional cover; convert pstar from mb to Pa, and winds from km/min
c     to m/s
c
          sum = 0.
          do 60 k = 1,nz
            dsigma = sigma(k) - sigma(k+1)
            if (lstagr) then
              uip = 16.67*u(i,j,k)
              uim = 16.67*u(i-1,j,k)
              vjp = 16.67*v(i,j,k)
              vjm = 16.67*v(i,j-1,k)
              vmid = 16.67*(v(i,j,k) + v(i,j-1,k))/2.
            else
              uip = 16.67*(u(i,j,k) + u(i+1,j,k))/2.
              uim = 16.67*(u(i,j,k) + u(i-1,j,k))/2.
              vjp = 16.67*(v(i,j,k) + v(i,j+1,k))/2.
              vjm = 16.67*(v(i,j,k) + v(i,j-1,k))/2.
              vmid = 16.67*v(i,j,k)
            endif
            pqip = 100.*pstar(i,j)*ch2o(i,j,k)
            pqim = pqip
            pqjp = pqip
            pqjm = pqip
            if (uip.lt.0.) pqip = 100.*pstar(i+1,j)*ch2o(i+1,j,k)
            if (uim.ge.0.) pqim = 100.*pstar(i-1,j)*ch2o(i-1,j,k)
            if (vjp.lt.0.) pqjp = 100.*pstar(i,j+1)*ch2o(i,j+1,k)
            if (vjm.ge.0.) pqjm = 100.*pstar(i,j-1)*ch2o(i,j-1,k)
            qdivu = (pqip*uip - pqim*uim)/(1000.*deltax(j))
            qdivv = (pqjp*vjp - pqjm*vjm)/(1000.*deltay)
            qdivlat = -100.*pstar(i,j)*ch2o(i,j,k)*vmid*tphioa(j)/1000.
            qdiv = qdivu + qdivv + qdivlat
            sum = sum - qdiv*dsigma
c           write(*,*)'qci,pui,qcim1,puim1'
c           write(*,*) qci,pui,qcim1,puim1
c           write(*,*)'qcj,pvj,qcjm1,pvjm1'
c           write(*,*) qcj,pvj,qcjm1,pvjm1
c           write(*,*)'qdivu,qdivv ',qdivu,qdivv,sum
   60     continue
c
c-----No convection; insufficient moisture convergence or cloud clover
c
          if (sum.lt.3.e-7) then
            if (lprnt) write(*,*)'Insufficient moisture convergence: ',
     &                 sum
            kbot = 0
            ktop = 0
          else
            cd = (1. - bfac)*sum/4.3e-3
            if (cd.gt.1.) cd = 1.
            if (cd.lt.0.05) then
              if (lprnt) write(*,*)'Insufficient cloud cover: ',
     &                   1.-bfac,cd
              cd = 0.
              kbot = 0
              ktop = 0
            else
c
c-----Convection is diagnosed!
c
              if (lprnt) then
                write(*,*)'Convection is happening!'
                write(*,*)'Moisture convergence: ',sum
                write(*,*)'Convective cloud coverage: ',cd
              endif
              if (kfreez.gt.1) kfreez = kfreez + 1
              goto 75
            endif
          endif
c
c-----Find vertical distribution of precipitating stratus if convection not
c     occurring.  Ensure the following:
c       1) the lowest cloud deck found is responsible for rain
c       2) average cloud coverage for the deck is at least 10%, otherwise 
c          no rain
c       3) the depth of the deck is greater than 0.1 (in sigma coords, which 
c          translates to about 1000 m in the lowest model layers).
c       4) the bottom of the deck is no higher than 3000 m AGL
c
   61     continue
          kbot = 0
          ktop = 0
          csav = 0.
          if (csmax.lt.0.1) then
             rain(i,j) = 0.
             goto 75
          endif
          do 70 k = 1,nz
            if (kbot.eq.0 .and. cs(k).ge.0.5*csmax) kbot = k
            if (kbot.ne.0 .and. cs(k).lt.0.5*csmax) then
              ktop = k - 1
              goto 71
            endif
   70     continue
          ktop = nz
   71     continue
          do 73 k = kbot,ktop
            dsigma = sigma(k) - sigma(k+1)
            csav = csav + cs(k)*dsigma
   73     continue
          sigtot = sigma(kbot) - sigma(ktop+1)
          csav = csav/sigtot
          if (sigtot.lt.0.1 .or. kbot.eq.ktop) then
             if (kbot.le.2) then
                cs(ktop+1) = cs(ktop)
             else
                cs(kbot-1) = cs(kbot)
             endif
             goto 61
          endif
          zbot = 0.
          if (kbot.gt.1) zbot = zi(kbot-1)
          if (csav.lt.0.1 .or. zbot.gt.3000.) then
            csav = 0.
            kbot = 0
            ktop = 0
            rain(i,j) = 0.
          endif
          if (lprnt) then
            write(*,*)'Top at   : ',ktop,sigma(ktop+1)
            write(*,*)'Bottom at: ',kbot,sigma(kbot)
          write(*,*)'Height of freezing: ',kfreez,sigma(kfreez)
            write(*,*)'Average stratus coverage: ',csav
          endif
   75     continue
c
c-----Load common variables for use in cloud sub-model
c
          kcldb(i,j) = kbot
          kcldt(i,j) = ktop
          kfrz(i,j) = kfreez
          fconv(i,j) = cd
          fstrt(i,j) = csav
          do 80 k = 1,nz
            clw(k) = clh2o(i,j,k)
            rlw(k) = rnh2o(i,j,k)
c           cldmax = 0.
c           do 85 kk = k,nz
c             cldmax = amax1(cldmax,cs(kk))
c  85       continue
c           cloud(i,j,k) = cldmax
             if (k.ge.kbot .and. k.le.ktop) then
                if (cd.ne.0.) then
                   cloud(i,j,k) = amax1(cd,cs(k))
                   if (cd .ge. cs(k)) cfnonpr(i,j,k) = 0.0
                   if (cd .lt. cs(k)) cfnonpr(i,j,k) = 1.0-(cd/cs(k))
                else
                   cloud(i,j,k) = amax1(csav,cs(k))
                   if (csav .ge. cs(k)) cfnonpr(i,j,k) = 0.0
                   if (csav .lt. cs(k)) cfnonpr(i,j,k) = 1.0-(csav/cs(k))
                endif
             else
                cloud(i,j,k) = cs(k)
                cfnonpr(i,j,k) = 1.0
             endif
c             write (*,*) 'clddiag', i,j,k,cfnonpr(i,j,k)
 80       continue 
c
c-----Estimate wet scavenging rates within precipitating clouds
c
          if (lwet .and. rain(i,j).ge.0.01) then
	   if ( cd .gt. 0.0 ) lcd=.true.
           call scavrat(lprnt,nz,rain(i,j),clw,rlw,
     &                  lcd,kbot,ktop,kfreez,zi,rate)
           if ( lprnt ) write(*,*) ' finished scavrat .....'
            do 90 k = 1,nz
              wrat(i,j,k) = rate(k)
   90       continue
          else
            do 91 k = 1,nz
              wrat(i,j,k) = 0.
   91       continue
          endif
c
  200   continue
  100 continue

      return
      end
