c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine ffgtrp(nx,ny,nz,acrs,ifget,
     &                  mxf,myf,nzfin,afine,sigf,ifine,itype,idebug)
c
c-----ATDM v1.00
c
c     Interpolates fine grid data to another fine grid as selected by ifine.
c
c-----95/03/31:
c     -from UAM-V Revision 1.1  94/05/17  17:22:48  tcm
c     -modifications made following UAM-V to correct vertical nesting code
c-----95/06/27:
c     -Replaced htf with sigf in calling arguments
c     -Modified vertical interpolation to use sigmaf as vertical
c      coordinate rather than physical height (htf)
c     -Fixed vertical interpolation to layer midpoints...
c      weighting factors and sigma's determined by values at
c      midpoints not layer interface values
c-----95/09/13:
c     -fixed vertical interpolation to layer interfaces (itype=0x) to
c      assume a value of zero at the surface
c
c     In comments below, coarse refers to the coarser of the two fine
c     grids, i.e., the one being interpolated from.
c
c     ones digit of itype selects type of interpolation for horizontal:
c        = 0 will result in a bilinear interpolation
c        = 1 will do a bilinear interpolation and then adjust values
c          such that the average of all fine grid values in a coarse grid cell
c          is equal to the value in the coarse grid cell
c
c     tens digit of itype selects type of interpolation for vertical:
c        = 0 assumes values at layer interfaces, linear interpolation
c            to additional layer interfaces, assume same value
c            coarse grid layer one top down to surface.
c        = 1 assumes values at layer middle, linear interpolation to
c            get values at new layer middles, assume same value from
c            middle of coarse grid layer one down to surface.
c        = 2 means that the value of nzfin is not actually vertical
c            levels and vertical interpolation should therefore be
c            skipped
c
c     Input arguments:
c               nx:             number of coarse cells in x-direction
c               ny:             number of coarse cells in y-direction
c               nz:             number of coarse grid layers
c               acrs:           coarse array to be interpolated
c               ifget:          index of coarser fine grid
c               mxf:            maximun number of cells in x-direction (fine)
c               myf:            maximun number of cells in y-direction (fine)
c               nzfin:          number of fine grid layers
c               sigf:           array of fine grid layer sigma's
c               ifine:          fine grid index
c               itype:          interpolation type
c               idebug:         diagnostic debug flag
c     Output arguments
c               afine           nterpolated values on fine grid
c     Files written:
c               6:              standard output file unit (*)
c     Routines called:
c               reducr
c     Called by:
c               update
c
      include 'parm.cmd'
      include 'parfin.cmd'
      include 'deffin.cmd'
      dimension acrs(nx,ny,nz), afine(mxf,myf,nzfin)
      dimension sigf(nzfin+1)
c
c  local arrays to ease the pain of the vertical interpolation
c
      dimension vcrs(mxfz), vfine(mxfz), scv(mxz+1)
c
c  fine grids include an extra row/column on top,left, 
c  bottom, and right of fine cells to allow for boundary
c  concentrations.  Simulated area is therefore exactly an
c  integral number of coarse grid cells.  
c
c  Bit two of idebug on (i.e., the twos location) will turn
c  on debug writes in ffgtrp.
c
      idiag = and(idebug,2)
c
c   set ratios of resolution for horizontal and vertical
c   and beginning of interpolated grid to source of data
c
      stpfin = 1./float(nhf(ifine))
      stpget = 1./float(nhf(ifget))
      ixff0 = (ixfb(ifine) - ixfb(ifget))*nhf(ifget) + 2
      jyff0 = (jyfb(ifine) - jyfb(ifget))*nhf(ifget) + 2
      if (ixff0 .lt. 2 .or. jyff0 .lt. 2 ) then
        write(*,*) ' erroneous grid parameters in ffgtrp'
        write(*,*) ' ixff0, jyff0: ',ixff0, jyff1
        write(*,*) 'ifine, ifget: ',ifine,ifget
        stop
      endif
c
c
c  split up itype into vertical and horizontal values
c
      ihtype = mod(itype,10)
      ivtype = itype/10
c
c  calculate interpolation factors (nhf of them)
c
c
      if (idiag .ne. 0) then
        write(*,'(a,2i5)') 'ihtype, ivtype: ',ihtype,ivtype
      endif
c
c  first do horizontal interpolation at each level
c  (note that we leave gaps between levels on the fine
c  grid to interpolate vertically later)
c 
      do 100 kc=1,nz
        if (ivtype .eq. 2) then
          kf = kc
        else
          kf = kvmap(kc,ifine)
        endif
c
        jy1 = (jyfb(ifine)-jyfb(ifget))*nhf(ifget)+1
        ycrs1 = float(jyfb(ifget)-1) + (float(jy1)-1.5)*stpget
        jy2 = jy1 + 1
        ycrs2 = ycrs1 + stpget
c
        do 60 j=1,nfy(ifine)
          yfin = float(jyfb(ifine)-1) + (float(j)-1.5)*stpfin
          if (yfin .gt. ycrs2) then
            jy1 = jy2
            ycrs1 = ycrs2
            jy2 = jy1 + 1
            ycrs2 = ycrs1 + stpget
          endif
          fy2 = (yfin-ycrs1)/stpget
          fy1 = 1. - fy2
c
          ix1 = (ixfb(ifine)-ixfb(ifget))*nhf(ifget)+1
          xcrs1 = float(ixfb(ifget)-1) + (float(ix1)-1.5)*stpget
          ix2 = ix1 + 1
          xcrs2 = xcrs1 + stpget
c
          do 50 i=1,nfx(ifine)
c
            xfin = float(ixfb(ifine)-1) + (float(i)-1.5)*stpfin
            if (xfin .gt. xcrs2) then
              ix1 = ix2
              xcrs1 = xcrs2
              ix2 = ix1 + 1
              xcrs2 = xcrs1 + stpget
            endif
            fx2 = (xfin-xcrs1)/stpget
            fx1 = 1. - fx2
c
            x1 = acrs(ix1,jy1,kc)*fy1 + acrs(ix1,jy2,kc)*fy2
            x2 = acrs(ix2,jy1,kc)*fy1 + acrs(ix2,jy2,kc)*fy2
            afine(i,j,kf) = x1*fx1 + x2*fx2
c
   50     continue
   60   continue
  100 continue
c
c  adjust interpolated data to average to coarse cell value
c  (if ihtype = 1)
c 
      if (ihtype .eq. 1) then
c
        call reducr(nhf(ifine),nhf(ifget),ifstep,igstep)
        ff2i = 1./float(ifstep*ifstep)
        fg2i = 1./float(igstep*igstep)
        do 290 kc=1,nz
          if (ivtype .eq. 2) then
            kf = kc
          else
            kf = kvmap(kc,ifine)
          endif
          if (idiag .ne. 0) then
            write(*,'(a,4i5)') 'kc, kf, nadd, nz: ',kc,kf,nadd,nz
          endif
          do 260 jbf=2,nfy(ifine)-1,ifstep
            jef = jbf + ifstep - 1
            do 250 ibf=2,nfx(ifine)-1,ifstep
              ief = ibf + ifstep - 1
              sum = 0.
              do 210 jf=jbf,jef
                do 205 if=ibf,ief
                  sum = sum + afine(if,jf,kf)
  205           continue
  210         continue
              avg = sum*ff2i
              if (avg .ne. 0.) then
                ig2 = (ief-1)*igstep/ifstep + ixff0 - 1
                ig1 = ig2 - igstep + 1
                jg2 = (jef-1)*igstep/ifstep + jyff0 - 1
                jg1 = jg2 - igstep + 1
                sum = 0.
                do 220 j=jg1,jg2
                  do 215 i=ig1,ig2
                    sum = sum + acrs(i,j,kc)
  215             continue
  220           continue
                avgg = sum*fg2i
                fcor = avgg/avg
              else
                fcor = 1.
              endif
              if (idiag .ne. 0) then
                write(*,'(a,e12.5,2x,e12.5,2x,e12.5,2x,e12.5)')
     &              'sum, fn2i, avg, fcor: ', sum, fn2i, avg, fcor
                write(*,'(a,3i5,e12.5)') 'i, j, kc, acrs: ',
     &              i, j, kc, acrs(i,j,kc)
              endif
              do 240 jf=jbf,jef
                do 230 if=ibf,ief
                  afine(if,jf,kf) = fcor*afine(if,jf,kf)
                  if (idiag .ne. 0) then
                    if (if .gt. 2 .and. if .lt. 5 .and.
     &                  jf .gt. 2 .and. jf .lt. 5) then
                      write(*,'(a,3i5,e12.5)') 'if,jf,kf,afine: ',
     &                      if, jf, kf, afine(if,jf,kf)
                    endif
                  endif
  230           continue
  240         continue
  250       continue
  260     continue
  290   continue
c
      endif
c
c   write out first few values of both coarse grid and fine
c   grid for diagnostics
c
      if (idiag .ne. 0) then
        write(*,'(a,5e11.4,/,a,5e11.4,/,a,10e11.4,/,a,10e11.4)')
     &      ' Coarse:  ',
     &      (acrs(i,jyff0,1),i=ixff0,ixff0+4),
     &      '          ',
     &      (acrs(i,jyff0+1,1),i=ixff0,ixff0+4),
     &      ' Fine:  ',
     &      (afine(i,2,1),i=2,11),
     &      '        ',
     &      (afine(i,3,1),i=2,11)
      endif
c
      if (nzfin .eq. nz) return
c
c  interpolate to tops of intervening layers
c
      if (ivtype .eq. 0) then
        kf = 0
        do 350 kc=1,nz
          f1 = 0.
          kflow = kf
          kf = kvmap(kc,ifine)
          nl = kf - kflow - 1
          do 330 nn=1,nl
            kkf = kflow + nn
	    f1 = (sigf(kflow+1)-sigf(kkf+1))/(sigf(kflow+1)-sigf(kf+1))
            f0 = 1. - f1
            do 320 jf=1,nfy(ifine)
              do 310 if=1,nfx(ifine)
                if (kc .gt. 1) then
                  a1 = afine(if,jf,kflow)
                else
                  a1 = 0.
                endif
                afine(if,jf,kkf) = a1*f0 + afine(if,jf,kf)*f1
  310         continue
  320       continue
  330     continue
  350   continue
c
      else if (ivtype .eq. 1) then
c
c   interpolate to middles of layers
c
        do 470 j=1,nfy(ifine)
          do 460 i=1,nfx(ifine)
            do 410 k=1,nz
              kf = kvmap(k,ifine)
              vcrs(k) = afine(i,j,kf)
              if (k .gt. 1) then
                kfm1 = kvmap(k-1,ifine)
              else
                kfm1 = 0
              endif
              scv(k) = 0.5*(sigf(kf+1)+sigf(kfm1+1))
  410       continue
            do 415 kf = 1,kvmap(1,ifine)
              vfine(kf) = vcrs(1)
  415       continue
            if (nz .le. 1) go to 460
            do 420 kf=kvmap(nz-1,ifine)+1,nzfin
              vfine(kf) = vcrs(nz)
  420       continue
            do 440 k=2,nz
              if (k .gt. 2) then
                kfbeg = kvmap(k-2,ifine) + 1
              else
                kfbeg = 1
              endif
              do 430 kf=kfbeg,kvmap(k,ifine)
		svm = 0.5 * (sigf(kf+1) + sigf(kf))
		if (svm .lt. scv(k)) goto 430
		if (svm .ge. scv(k-1)) goto 430
                f1 = (scv(k-1)-svm)/(scv(k-1)-scv(k))
                f0 = 1. - f1
                vfine(kf) = f0*vcrs(k-1) + f1*vcrs(k)
  430         continue
  440       continue
            do 450 kf=1,nzfin
              afine(i,j,kf) = vfine(kf)
  450       continue
  460     continue
  470   continue
c
      else
c
        return
c
      endif
      if (idiag .ne. 0) then
        write(*,'(a,5e11.4,/,a,5e11.4,/,a,10e11.4,/,a,10e11.4)')
     &      ' After vert, Coarse:  ',
     &      (acrs(i,jyff0,1),i=ixff0,ixff0+4),
     &      '                      ',
     &      (acrs(i,jyff0+1,1),i=ixff0,ixff0+4),
     &      ' After vert, Fine:  ',
     &      (afine(i,2,1),i=2,11),
     &      '                    ',
     &      (afine(i,3,1),i=2,11)
      endif
c
      return
      end
