c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine fgrtrp(mxc,myc,nx,ny,nz,acrs,mxf,myf,nzfin,afine,sigf,
     &                  ifine,itype,its,tsurf,ustgr,vstgr,idebug)
c
c-----ATDM v1.00
c
c     Interpolates coarse grid data onto a fine grid selected by ifine.
c
c-----95/06/02:
c     -from UAM-V Revision 1.2  94/02/10  20:55:48  tcm
c-----95/06/26 (GEM):
c     -Replaced htf with sigf in calling arguments
c      Modified vertical interpolation to use sigmaf as vertical
c      coordinate rather than physical height (htf)
c     -Fixed vertical interpolation to layer midpoints...
c      weighting factors and sigma's determined by values at 
c      midpoints not layer interface values
c-----95/09/05:
c     -Added ITS flag and Ts array to argument list; if its=1, then vertical
c      interpolation will use Ts rather than constant temperature below
c      midpoint of first layer.  Ts=0 and a dummy argument are passed for all
c      other calls to FGRTRP
c-----95/09/13:
c     -fixed vertical interpolation to layer interfaces (itype=0x) to 
c      assume a value of zero at the surface  
c-----95/09/19:
c     -fixed horizontal interpolation to handle staggered winds properly
c-----95/12/01:
c     -fixed bug in TS interpolation (see note 95/09/05)
c     -Good Bye
c
c     ones digit of itype selects type of interpolation for horizontal:
c        = 0 will result in a bilinear interpolation
c        = 1 will do a bilinear interpolation and then adjust values
c            such that the average of all fine grid values in a coarse 
c            grid cell is equal to the value in the coarse grid cell
c
c     tens digit of itype selects type of interpolation for vertical:
c        = 0 assumes values at layer interfaces, linear interpolation
c            to additional layer interfaces, assume 0. at surface.
c        = 1 assumes values at layer middle, linear interpolation to
c            get values at new layer middles, assume same value from
c            middle of coarse grid layer one down to surface.
c        = 2 means that the value of nzfin is not actually vertical
c            levels and vertical interpolation should therefore be
c            skipped
c
c     Input arguments:
c               mxc:            maximum number of cells in x-direction (coarse)
c               myc:            maximum number of cells in y-direction (coarse)
c               nx:             number of coarse cells in x-direction
c               ny:             number of coarse cells in y-direction
c               nz:             number of coarse grid layers
c               acrs:           coarse array to be interpolated
c               mxf:            maximum number of cells in x-direction (fine)
c               myf:            maximum number of cells in y-direction (fine)
c               nzfin:          number of fine grid layers
c               sigf:           array of fine grid layer sigma's
c               ifine:          fine grid index
c               itype:          interpolation type
c               its:            vertical temperature interpolation flag
c               tsurf:          2-D surface temperature array (its=1)
c                               or dummy argument (its=0)
c               ustgr:          u-component staggered wind flag
c               vstgr:          v-component staggered wind flag
c               idebug:         diagnostic debug flag
c     Output arguments
c               afine           nterpolated values on fine grid
c     Files written:
c               6:              standard output file unit (*)
c     Called by:
c               atdm      update
c
      include 'parfin.cmd'
      include 'parm.cmd'
      include 'deffin.cmd'
      dimension acrs(mxc,myc,nz), afine(mxf,myf,nzfin)
      dimension sigf(nzfin+1)
      dimension tsurf(mxf,myf)
      dimension ffx0(maxdiv),ffx1(maxdiv),ffy0(maxdiv),ffy1(maxdiv)
      logical ustgr,vstgr
c
c  local arrays to ease the pain of the vertical interpolation
c
      dimension vcrs(mxz), vfine(mxfz), scv(mxz+1)
c
c  fine grids include an extra row/column on top,left, 
c  bottom, and right of fine cells to allow for boundary
c  concentrations.  Simulated area is therefore exactly an
c  integral number of coarse grid cells.  niggle is a value
c  that tells the number of fine cells that we are short of
c  being at the mid point of a coarse cell at the start of the
c  overall fine grid.  It is used to help pick the correct cell
c  cells to use in interpolation.
c
c  Bit two of idebug on (i.e., the twos location) will turn
c  on debug writes in fgrtrp.
c
      idiag = and(idebug,2)
c
      niggle = 1+(nhf(ifine)+1)/2
c
      if (idiag .ne. 0) then
        write(*,'(a,i4)') ' niggle: ',niggle
      endif
c
c  split up itype into vertical and horizontal values
c
      ihtype = mod(itype,10)
      ivtype = itype/10
c
c  calculate interpolation factors (nhf of them)
c
      if (mod(nhf(ifine),2) .eq. 0) then
        ffx1(1) = 0.5/float(nhf(ifine))
        ffy1(1) = 0.5/float(nhf(ifine))
      else
        ffx1(1) = 1./float(nhf(ifine))
        ffy1(1) = 1./float(nhf(ifine))
      endif
      if (ustgr) ffx1(1) = 0.
      if (vstgr) ffy1(1) = 0.
      ffx0(1) = 1. - ffx1(1)
      ffy0(1) = 1. - ffy1(1)
c
      if (idiag .ne. 0) then
        write(*,'(a,2i5)') 'ihtype, ivtype: ',ihtype,ivtype
        write(*,'(a,2e11.4)') ' ff0   ff1: ',ff0(1),ff1(1)
      endif
c
      do 20 n=2,nhf(ifine)
        ffx1(n) = ffx1(n-1) + 1./float(nhf(ifine))
        ffy1(n) = ffy1(n-1) + 1./float(nhf(ifine))
        ffx0(n) = 1. - ffx1(n)
        ffy0(n) = 1. - ffy1(n)
c
        if (idiag .ne. 0) then
          write(*,'(a,2e11.4)') ' ff0   ff1: ',ff0(n),ff1(n)
        endif
c
   20 continue
c
c  first do horizontal interpolation at each level
c  (note that we leave gaps between levels on the fine
c  grid to interpolate vertically later)
c 
      do 100 kc=1,nz
        if (ivtype .eq. 2) then
          kf = kc
        else
          kf = kvmap(kc,ifine)
        endif
        do 60 j=jyfb(ifine),jyfe(ifine)+1
          if (vstgr .and. mod(nhf(ifine),2).eq.0) then
            jef = (j - jyfb(ifine))*nhf(ifine) + niggle + 1
          else
            jef = (j - jyfb(ifine))*nhf(ifine) + niggle
          endif
          jbf = max(1,jef-nhf(ifine)+1)
          do 50 i=ixfb(ifine),ixfe(ifine)+1
            if (ustgr .and. mod(nhf(ifine),2).eq.0) then
              ief = (i - ixfb(ifine))*nhf(ifine) + niggle + 1
            else
              ief = (i - ixfb(ifine))*nhf(ifine) + niggle
            endif
            ibf = max(1,ief-nhf(ifine)+1)
            do 40 jf=jbf,jef
              if (jf .gt. nfy(ifine)) go to 40
              nn = nhf(ifine) - jef + jf
              x0 = acrs(i-1,j-1,kc)*ffy0(nn) + acrs(i-1,j,kc)*ffy1(nn)
              x1 = acrs(i,j-1,kc)*ffy0(nn) + acrs(i,j,kc)*ffy1(nn)
              do 30 if=ibf,ief
                if (if .gt. nfx(ifine)) go to 30
                nn = nhf(ifine) - ief + if
                afine(if,jf,kf) = x0*ffx0(nn) + x1*ffx1(nn)
c
                if (idiag .ne. 0) then
                  if ( (i .eq. ixfb(ifine) .or. i .eq. ixfb(ifine)+1)
     &                .and.
     &                (j .eq. jyfb(ifine) .or. j .eq. jyfb(ifine)+1) )
     &                then
                    write(*,'(a,/,1x,i4,5e11.4,/,5x,a,/,5x,11i4)') 
     &               ' nn, ff0, ff1, x0, x1, afine',
     &               nn, ff0(nn), ff1(nn), x0, x1, afine(if,jf,kf),
     &               'ibf, ief, jbf, jef, if, jf, kf, i, j, kc, ifine',
     &               ibf, ief, jbf, jef, if, jf, kf, i, j, kc, ifine
                  endif
                endif
c
   30         continue
   40       continue
   50     continue
   60   continue
  100 continue
c
c  adjust interpolated data to average to coarse cell value
c  (if ihtype = 1)
c 
      if (ihtype .eq. 1) then
c
        fn2i = 1./float(nhf(ifine)*nhf(ifine))
        do 290 kc=1,nz
          if (ivtype .eq. 2) then
            kf = kc
          else
            kf = kvmap(kc,ifine)
          endif
          if (idiag .ne. 0) then
            write(*,'(a,4i5)') 'kc, kf, nadd, nz: ',kc,kf,nadd,nz
          endif
          do 260 j=jyfb(ifine),jyfe(ifine)
            jef = (j - jyfb(ifine) + 1)*nhf(ifine) + 1
            jbf = jef-nhf(ifine)+1
            do 250 i=ixfb(ifine),ixfe(ifine)
              ief = (i - ixfb(ifine) + 1)*nhf(ifine) + 1
              ibf = ief-nhf(ifine)+1
              sum = 0.
              do 220 jf=jbf,jef
                do 210 if=ibf,ief
                  sum = sum + afine(if,jf,kf)
  210           continue
  220         continue
              avg = sum*fn2i
              if (avg .ne. 0.) then
                fcor = acrs(i,j,kc)/avg
              else
                fcor = 1.
              endif
              if (idiag .ne. 0) then
                write(*,'(a,e12.5,2x,e12.5,2x,e12.5,2x,e12.5)')
     &              'sum, fn2i, avg, fcor: ', sum, fn2i, avg, fcor
                write(*,'(a,3i5,e12.5)') 'i, j, kc, acrs: ',
     &              i, j, kc, acrs(i,j,kc)
              endif
              do 240 jf=jbf,jef
                do 230 if=ibf,ief
                  afine(if,jf,kf) = fcor*afine(if,jf,kf)
                  if (idiag .ne. 0) then
                    if (if .gt. 2 .and. if .lt. 5 .and.
     &                  jf .gt. 2 .and. jf .lt. 5) then
                      write(*,'(a,3i5,e12.5)') 'if,jf,kf,afine: ',
     &                      if, jf, kf, afine(if,jf,kf)
                    endif
                  endif
  230           continue
  240         continue
  250       continue
  260     continue
  290   continue
c
      endif
c
c   write out first few values of both coarse grid and fine
c   grid for diagnostics
c
       if (idiag .ne. 0) then
        write(*,*) 'First few crs & fine values after interpol within frgtrp'
        write(*,'(a,5e11.4,/,a,5e11.4,/,a,10e11.4,/,a,10e11.4)')
     &      ' Coarse:  ',
     &      (acrs(i,jyfb(ifine),1),i=ixfb(ifine),ixfb(ifine)+4),
     &      '          ',
     &      (acrs(i,jyfb(ifine)+1,1),i=ixfb(ifine),ixfb(ifine)+4),
     &      ' Fine:  ',
     &      (afine(i,2,1),i=2,11),
     &      '        ',
     &      (afine(i,3,1),i=2,11)
       endif
c
      if (nzfin .eq. nz) return
c
c  interpolate to tops of intervening layers
c
      if (ivtype .eq. 0) then
        kf = 0
        do 350 kc=1,nz
          f1 = 0.
          kflow = kf
          kf = kvmap(kc,ifine)
          nl = kf - kflow - 1
          do 330 nn=1,nl
            kkf = kflow + nn
	    f1 = (sigf(kflow+1)-sigf(kkf+1))/(sigf(kflow+1)-sigf(kf+1))
            f0 = 1. - f1
            do 320 jf=1,nfy(ifine)
              do 310 if=1,nfx(ifine)
                if (kc .gt. 1) then
                  a1 = afine(if,jf,kflow)
                else
                  a1 = 0.
                endif
                afine(if,jf,kkf) = a1*f0 + afine(if,jf,kf)*f1
  310         continue
  320       continue
  330     continue
  350   continue
c
      else if (ivtype .eq. 1) then
c
c   interpolate to middles of layers
c
        do 470 j=1,nfy(ifine)
          do 460 i=1,nfx(ifine)
            do 410 k=1,nz
              kf = kvmap(k,ifine)
              vcrs(k) = afine(i,j,kf)
              if (k .gt. 1) then
                kfm1 = kvmap(k-1,ifine)
              else
                kfm1 = 0
              endif
              scv(k) = 0.5*(sigf(kf+1)+sigf(kfm1+1))
  410       continue
            do 415 kf = 1,kvmap(1,ifine)
              vfine(kf) = vcrs(1)
  415       continue
            if (nz .le. 1) go to 460
            do 420 kf=kvmap(nz-1,ifine)+1,nzfin
              vfine(kf) = vcrs(nz)
  420       continue
            do 440 k=2,nz
              if (k .gt. 2) then
                kfbeg = kvmap(k-2,ifine) + 1
              else
                kfbeg = 1
              endif
              do 430 kf=kfbeg,kvmap(k,ifine)
		svm = 0.5 * (sigf(kf+1) + sigf(kf))
		if (svm .lt. scv(k)) goto 430
                if (kfbeg.eq.1 .and. its.eq.1 .and.
     &              svm.gt.scv(k-1)) then
                  f1 = (svm-scv(k-1))/(sigf(1)-scv(k-1))
                  f0 = 1. - f1
                  vfine(kf) = f0*tsurf(i,j) + f1*vcrs(k-1)
c                 if (i.eq.21.and.j.eq.21.and.its.eq.1) then
c                    write(*,*)'k,kf,svm,scv-,scv+,f1,f0,tsurf,vcrs,
c    &                          fine '
c                    write(*,*)k,kf,svm,scv(k-1),scv(k),f1,f0,
c    &                         tsurf(i,j),vcrs(k-1),vfine(kf)
c                 endif
                endif
                if (svm .ge. scv(k-1)) goto 430
                f1 = (scv(k-1)-svm)/(scv(k-1)-scv(k))
                f0 = 1. - f1
                vfine(kf) = f0*vcrs(k-1) + f1*vcrs(k)
  430         continue
  440       continue
            do 450 kf=1,nzfin
              afine(i,j,kf) = vfine(kf)
  450       continue
  460     continue
  470   continue
c
      else
c
        return
c
      endif
      if (idiag .ne. 0) then
        write(*,'(a,5e11.4,/,a,5e11.4,/,a,10e11.4,/,a,10e11.4)')
     &      ' After vert, Coarse:  ',
     &      (acrs(i,jyfb(ifine),1),i=ixfb(ifine),ixfb(ifine)+4),
     &      '                      ',
     &      (acrs(i,jyfb(ifine)+1,1),i=ixfb(ifine),ixfb(ifine)+4),
     &      ' After vert, Fine:  ',
     &      (afine(i,2,1),i=2,11),
     &      '                    ',
     &      (afine(i,3,1),i=2,11)
      endif
c
      return
      end
