c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine sulfate(pm,so2av,rsulf,clwc0,tempr,pressr,
     &                 hf,deltat,lprnt,cloud,fnonpr)
c
c-----ATDM v6.00
c
c     Calculates transformation of SO2 to sulfate
c
c      Ref:
c      Seinfeld and Pandis
c      Martin (1984)
c      Martin (1994)
c            
c
c     Input arguments:
c               pm              concentration of 5 species (ppm)
c                               SO2, GSO4, ASO4, H2O2, O3
c               rsulf           incremental concentration of SULF
c                               from step4 (ppm)
c               clwc            cloud liquid water content
c               pressr          pressure (mb)
c               deltat          time step (min)
c               lprnt           diagnostic debug print flag 
c     Output arguments:
c               pm              concentration of 5 species (ppm)
c               so2av           average in cloud so2 (ppm)
c     Called by:
c               rchem2     rchem2f
c
c
c      save
      dimension pm(5), dcdt(4), rjc(3,3), cc(3)
      real*4 hf(3)
      logical lprnt
      logical lfirst
      save lfirst, nchk, clavg, fnonav, n0cl, n0fn
c
c  rkh2o2 from Martin 1984, Figure 21 @ ph4.5
c  rko3 from Martin 1984, expression on p 86, assume ph4.5
c  rko2 calculated to fit %/hr conversion in Fig 30, Martin 1994
c     data rkh2o2, rko3, rko2 /2.3e3, 3.6e6 , 3.2e+0/
      data rkh2o2a, rko3a, rko3b, rko2a /2.3e3, 4.19e5, 1.e2, 3.2e+0/
      data acth2o2a, acto3a, acto3b, acto2a /4026.,4126.,956.,13738./
      data R /0.082/
c  temporary Henry's law coefficients for testing
c     data h1, h2, h3, h4 /1.23  , 7.10E+4 , 1.13E-2, 1.e-3/ 
      data h1, h2, h3, h4 /7.e2  , 7.10E+4 , 1.13E-2, 1.e-3/ 
      data lfirst /.true./
c
      if (lfirst) then
        lfirst = .false.
        nchk = 0
        clavg = 0.
        fnonav = 0.
        n0cl = 0
        n0fn = 0
      endif
c
      press = pressr/1013.
      clwc = clwc0
c     cldmin = cloud*fnonpr
      cldmin = cloud
      if (cldmin .eq. 0.) n0cl = n0cl + 1
      if (fnonpr .eq. 0.) n0fn = n0fn + 1
c
      h1 = hf(1)
      h2 = hf(3) 
      h3 = hf(2)
c
      tterm = 1./298. - 1./tempr
      rkh2o2 = rkh2o2a*exp(acth2o2a*tterm)
      rko2 = rko2a*exp(acto2a*tterm)
      rko3 = rko3a*exp(acto3a*tterm) 
     &               + rko3b/(10.**-4.5)*exp(acto3b*tterm)
c     if (lprnt) then
c       write(*,*) 'rkh2o2t, rkh2o2, acth2o2a: ', rkh2o2t, rkh2o2, 
c    &                                   acth2o2a
c       write(*,*) 'rko2t, rko2, acto2: ', rko2t, rko2, acto2
c       write(*,*) 'rko3t, rko3, acto3a&b: ', rko3t,rko3,acto3a,acto3b
c     endif
c
      if (lprnt) then
        write(*,*)'sulfate in: SO2,GSO4,ASO4,H2O2,O3,rsulf,clwc,cldmin'
        write(*,*)(pm(l),l=1,5),rsulf,clwc,cldmin
      endif
      s0 = pm(1)
c
c-----Sulfate chemistry combines slow heterogeneous reactions (bsulf),
c     aqueous reactions (wsulf) and OH reactions (rsulf).  Sulfate via
c     OH channel is calculated in step4 of gas-phase chemistry. In
c     this routine, the GSO4 is updated by adding the incremental conc.
c     of SULF. 
c
c     tstp=deltat/60.
c     bsulf = 0.002*tstp
c     bsulf = pm(1)*bsulf
c
c-----If liquid water content is less than cutoff, skip aqueous
c     conversion of SO2 to sulfate.
c
c     if (clwc.lt.1.e-9 .or. cldmin .lt. 0.05) then
      if (cldmin .lt. 0.05) then
        wsulf=0.
        so2av = 0.
      else
        if (clwc .lt. 1.e-11) clwc = 1.e-11
        clavg = clavg + cldmin
        fnonav = fnonav + fnonpr
        nchk = nchk + 1
        if (mod(nchk,10000) .eq. 0) then
          ff = 1./float(nchk)
          fnonav = fnonav*ff
          clavg = clavg*ff
          write(*,*) 'Average cloud = ', clavg
          write(*,*) 'Average fnonpr = ', fnonav
          write(*,*) 'Number of zero cloud = ', n0cl
          write(*,*) 'Number of zero fnonpr = ', n0fn
          nchk = 0
          clavg = 0.
          fnonav = 0.
          n0cl = 0
          n0fn = 0
          lprnt = .true.
        endif
c
c-----rate of change of each species is hheff*partial pressure*
c     hheff(kso2)*partial pressure so2*clwc*chemical rate
c
c    calculate partial pressure of each species
c
      o3 = pm(5)*1.e-6*press
      h2o2 = pm(4)*1.e-6*press
      o2 = 0.2*press
      so2 = pm(1)*1.e-6*press
      cc(1) = so2
      cc(2) = h2o2
      cc(3) = o3
c
c   loop over time steps
c
      tnow = 0.
      so2av = cc(1)
      ndt = 1
      hdt = deltat/10.
c
   50   dcdt(2) = -60.*rkh2o2*h2*h2o2*h1*so2*R*tempr*clwc
        if (h2o2 .lt. 1.e-20) then
          dcdt(2) = 0.
          h2o2 = 1.e-20
        endif
        dcdt(3) = -60.*rko3*h3*o3*h1*so2*R*tempr*clwc
        if (o3 .lt. 1.e-20) then
          dcdt(3) = 0.
          o3 = 1.e-20
        endif
        dcdt(4) = -60.*rko2*h4*o2*h1*so2*R*tempr*clwc
        dcdt(1) = dcdt(2) + dcdt(3) + dcdt(4)
        if (so2 .lt. 1.e-20) then
          dcdt(1) = 0.
          dcdt(2) = 0.
          dcdt(3) = 0.
          so2 = 1.e-20
          go to 70
        endif
        rjc(1,1) = dcdt(1)/so2
        rjc(1,2) = dcdt(2)/h2o2
        rjc(1,3) = dcdt(3)/o3
        rjc(2,1) = dcdt(2)/so2
        rjc(2,2) = dcdt(2)/h2o2
        rjc(2,3) = 0.
        rjc(3,1) = dcdt(3)/so2
        rjc(3,2) = 0.
        rjc(3,3) = dcdt(3)/o3
        do i=1,3
          do j=1,3
            rjc(i,j)=-0.5*hdt*rjc(i,j)
          enddo
        enddo
        do i=1,3
          rjc(i,i)=1.+rjc(i,i)
        enddo
        do l=1,3
c         dcdt(l) = -(hold(l) - con(l) - hdt*rate(l))
          dcdt(l) = hdt*dcdt(l)
        enddo
c
        call colpivot(1,3,3,rjc,dcdt,ier)
c
        if (ier.eq.0) then
          lerr=.true.
          write(*,*) 'error in sulfate calculations'
          write(*,*) 'in colpivot, matrix of equations is singular'
          write(*,*) 'ier,iter,nitmax= ',ier,iter,nitmax
          write(*,*) (n,dcdt(n),n=1,3)
          write(*,*) ((n,m,rjc(n,m),n=1,3),m=1,3)
          write(*,*) 'cc: ',(cc(i),i=1,3)
          write(*,*) 'pm: ',(pm(i),i=1,5)
          write(*,*) o2, so2, o3, h2o2
          write(*,*) 'hf: ', (hf(i),i=1,3)
          write(*,*) 'tempr, press, clwc: ', tempr, press, clwc
          write(*,*) 'deltat: ', deltat
          stop
        endif
c
        do i=1,3
          cc(i) = cc(i) + dcdt(i)
          if (cc(i) .le. 0. .or. ier .eq. 0) then
            hdt = hdt*0.5
            cc(1) = so2
            cc(2) = h2o2
            cc(3) = o3
            go to 50
          endif
        enddo
        so2 = cc(1)
        so2av = so2av + cc(1)
        h2o2 = cc(2)
        o3 = cc(3)
        tnow = tnow + hdt
c       write(*,*) 'Time = ', tnow, ' min.'
c       write(*,*) 'so2, h2o2, o3: ', so2, h2o2, o3
c       write(*,*) 'dcdt: ', dcdt
        hdt = amin1(hdt,deltat-tnow)
        if (tnow .lt. 0.99*deltat) then
          ndt = ndt + 1
          go to 50
        endif
c
   70 if (lprnt) then
        write(*,*)'sulfate in: SO2,GSO4,ASO4,H2O2,O3,rsulf,clwc'
        write(*,*)(pm(l),l=1,5),rsulf,clwc
      endif
      so2av = so2av/float(ndt+1) * 1.e6/press
      wsulf = (pm(1) - so2*1.e6/press) * cldmin
      if (wsulf .gt. s0) wsulf = s0
      pm(1) = pm(1) - wsulf
      delh2o2 = (pm(4) - h2o2*1.e6/press) * cldmin
      pm(4) = pm(4) - delh2o2
      delo3 = (pm(5) - o3*1.e6/press) * cldmin
      pm(5) = pm(5) - delo3
c
      endif
c
c---- Combine heterogeneous sulfate (bsulf) and aqueous sulfate (wsulf).
c
c     sulfw = wsulf + bsulf
      sulfw = wsulf
c
c---- Check total sulfur balance.
c
      if (sulfw.gt.s0) sulfw = s0
c
c---- Calculating the new concentration for pm species.
c
      sulfg = rsulf
      pm(2) = pm(2) + sulfg
      pm(3) = pm(3) + sulfw
c
      if (lprnt) then
        write(*,*)'bsulf,wsulf,sulfw,sulfg'
        write(*,*) bsulf,wsulf,sulfw,sulfg 
        write(*,*) 'sulfate out: SO2,GSO4,ASO4,H2O2,O3,so2av'
        write(*,*) (pm(l),l=1,5),so2av
      endif
c
      lprnt=.false.
c
      return
      end
